#include "memory-address-configs.h"

    .set    noreorder
    .set    mips32
    .text

.macro WAIT_TBE
4:
    lw      $3, TSR0 ($2)
    andi    $3, $3, (1)
    beqz    $3, 4b
    nop
.endm

.macro WAIT_TRUN
5:
    lw      $3, TSR0 ($2)
    andi    $3, $3, TSR_TRUN
    bnez    $3, 5b
    nop
.endm

.macro WAIT_RBE
2:
    lw      $3, RSR0 ($2)
    andi    $3, $3, (1)
    bnez    $3, 2b
    nop
.endm

.macro RX_FLUSH
7:
    lw      $3, LRx0 ($2)
    lw      $3, RSR0 ($2)
    andi    $3, $3, (1)
    beqz    $3, 7b
    nop
.endm

_start: .global _start
    #la $6, 0x80000001
    #mfc0 $8,$15,1

boot_spi:
    # SPI Controller registers
    CR_PLL      =   (0xB82F4000-BASE_SPI)        #from BASE_SPI
    CLK_EN      =   (0xB82F4004-BASE_SPI)        #from BASE_SPI
    
    LTx0        =   0x00
    LRx0        =   0x00
    LCSR0       =   0x04
    LDIR0       =   0x08
    LDR0        =   0x0C
    TCTR0       =   0x10
    RCTR0       =   0x14
    TSR0        =   0x18
    RSR0        =   0x1C

    li      $2, BASE_SPI

#if defined(TARGET_MCT06) & !defined(SPI_BOOT2)
    # MCT-06 BootROM doesn't enable any CLK
    # despite of CPU CLK. That's why it necessary
    # to force set CLK_EN for MFBSP + CPU on first
    # SPL(BL1) stage!
    li      $7, 0x60001             # CPU + MFBSP
    sw      $7, CLK_EN($2)
#endif

    li      $10, 0xC1F8001B         # SS = 1
    
    # LCSR Setup
    ori     $4, $0, 0x200           # SPI Mode ON
    sw      $4, LCSR0 ($2)
    
#if !defined(SPI_BOOT2)
    
    # unset SS
    sw      $10, TCTR0 ($2)
    # Включение частоты порта MFBSP
    # Закомментировано, так как Bootrom включает частоту MFBSP
    # lw      $4, CLK_EN($2)
    # ori     $4, $4, 0x100
    # sw      $4, CLK_EN($2)
    #  LDIR0 Setup
    ori     $4, $0, 0x2F           # Pins setup
    sw      $4, LDIR0 ($2)
    # настройка частоты SCK
    #ori     $4, $0, 0x1001          
    #sw      $4,TCTR_RATE($2)
    #sw      $4,RCTR_RATE($2)
#else
#if defined(TCTR_RATE) && defined(RCTR_RATE)
    lw      $4, CR_PLL($2)
    sra     $4, $4, MFBSP_CLK_CR_PLL_OFF # CLK_SEL_PERIPH field
    andi    $4, $4, MFBSP_CLK_CR_PLL_MSK
    sw      $4, TCTR_RATE($2)
    sw      $4, RCTR_RATE($2)
#endif
    nop                             # filling 256 byte size
#endif

    # Read & Load Firmware image from flash
    la      $5, BASE_MEM
    li      $8, FIRM_BASE
    li      $9, FIRM_SIZE

    # send cmd_read | address    
#if defined(TCTR_RATE) && defined(RCTR_RATE)
    li      $4, 0x81F8001B          # frame 32bit, SS = 0
    sw      $4, TCTR0($2)
    li      $4, 0x01F8000F          # frame 32bit
    sw      $4, RCTR0($2)
#else
    li      $4, 0x81F801FB          # frame 32bit, tclk_rate = 0x1f, SS = 0
    sw      $4, TCTR0($2)
    li      $4, 0x01F801FF          # frame 32bit, rclk_rate = 0x1f
    sw      $4, RCTR0($2)
#endif

    lui     $6, 0x0300
    or      $6, $8                  # send 3-byte addr | READ_CMD
WAIT_TBE
    sw      $6, LTx0 ($2)
WAIT_TRUN
WAIT_RBE
    lw      $4, LRx0 ($2)

1:
WAIT_TBE
    sw      $0, LTx0 ($2)
WAIT_TRUN
WAIT_RBE
    lw      $11, LRx0 ($2)

    sw      $11, 0x0 ($5)           # store to CRAM
    addiu   $5, $5, 4
    addi    $9, $9, -4
    bnez    $9, 1b
    nop
    # unset SS
    sw      $10, TCTR0($2)

    la      $4, FIRM_ENTRY
    jr      $4
    nop
